/*
    libporty - async background processes for windows
    Copyright (C) 2010..2012  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page: http://repo.hu/projects/libporty
    Contact (email and IRC): http://igor2.repo.hu/contact.html
*/

#include "win32_loader/loader2.h" /* it has to be included before windows.h */
#include <stdio.h>
#include <libporty/bgtask/bgtask.h>

/*
TODO: check if it is a good idea
static void P_net_async_bgproc_set_inheritance(P_net_socket *inherit_list, int inherit_len)
{
	int n;

	for(n = 0; n < 1024; n++)
		SetHandleInformation(n, HANDLE_FLAG_INHERIT, 0);
}
*/


int P_net_async_bgproc_inherit(char *argv[], P_net_socket *ctrlfd, P_net_socket *inherit_list, int *child_fd, int inherit_len)
{
	int ret, argc, n, len, innerpid, param_size;
	PROCESS_INFORMATION pi;
	char *cmdline, *end;
	loader2_param_t *param;

/*	P_net_async_bgproc_set_inheritance(inherit_list, inherit_len); */

	for(argc = 0, len = 0; argv[argc] != NULL; argc++)
		len += strlen(argv[argc]) + 2;

	cmdline = malloc(len + 32);

	end = cmdline + sprintf(cmdline, "%s %s", P_SHELL, P_SHELL_QUOTE);

	for(n = 0; n < argc; n++)
		end += sprintf(end, "%s ", argv[n]);
	end += sprintf(end, "%s", P_SHELL_QUOTE);

	end--;
	*end = '\0';

/* Set up members of the PROCESS_INFORMATION structure. */
	ZeroMemory( &pi, sizeof(PROCESS_INFORMATION) );



	param_size = loader2_param_t_size(inherit_len);
	param = malloc(param_size);
	param->fdn = inherit_len;

	for(n = 0; n < inherit_len; n++) {
		param->fds[n].fd = child_fd[n];
		param->fds[n].handle = inherit_list[n];
		param->fds[n].mode = LOADER2_FD_MODE_TEXT;
	}

	ret = RunProcessWithFds(cmdline, param, param_size, &pi);

	free(cmdline);

	if (ctrlfd != NULL) {
		int cfd;
		P_net_socket inv = P_NET_INVALID_SOCKET;

		ret = P_net_local_socket(ctrlfd, &cfd, P_net_nonblock_full);
		if (ret != 0)
			return ret;

		watchdog_add(&cfd, &(pi.hProcess), &inv);
	}

	return 0;
}

#define STATUS_KILL 0xCC000001
#define STATUS_TERM 0xCC000002

static int watchdog_wait(P_net_socket cfd, int numpids, P_pid_t *pids_, bgproc_status_t *exit_status)
{
	int err;
	int *pids = pids_;
	int n, w;

	for(n = 0; n < numpids; n += MAXIMUM_WAIT_OBJECTS) {
		w = numpids - n;
		if (w > MAXIMUM_WAIT_OBJECTS)
			w = MAXIMUM_WAIT_OBJECTS;
/*		printf("watchdog waiting for %d pids starting from %d (all pids=%d), first pid: %d\n", w, n, numpids, (int)pids[n]);*/
		err = WaitForMultipleObjects(w, pids + n, 0, 0);
		if (err == WAIT_TIMEOUT) {
/*			printf(" timeout\n");*/
			return -1;
		}

		if ((err >= 0) && (err < WAIT_ABANDONED_0)) {
			err -= WAIT_OBJECT_0;
			GetExitCodeProcess(pids[err], &(exit_status->code));
			if ((exit_status->code == STATUS_TERM) || (exit_status->code == STATUS_KILL))
				exit_status->reason = BGPROC_KILLED;
			else if ((exit_status->code & 0xC0000000) == 0xC0000000)
				exit_status->reason = BGPROC_ABNORMAL;
			else
				exit_status->reason = BGPROC_EXITED;
/*			printf("watchdog EXITED: entry=%d pid=%d status=%d\n", err, pids[err], *exit_status);*/
			return err;
		}
/*
		else
			printf("  is %d\n", GetLastError());
*/
	}
	return -1;
}

static void watchdog_kill(P_pid_t pid, int hard)
{
	TerminateProcess(pid, hard ? STATUS_KILL : STATUS_TERM);
}
