/*
    libporty - async stdio
    Copyright (C) 2010..2012  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page: http://repo.hu/projects/libporty
    Contact (email and IRC): http://igor2.repo.hu/contact.html
*/

#include "astdio.h"
#include <libporty/bgtask/bgtask.h>

#ifdef P_NET_STDIO_BLOCKS
# include <unistd.h>
# include <libporty/net/local.h>
# include <assert.h>
	static P_net_socket stdin_sock = -1, stdout_sock = -1;

	/* thread for reading stdin in blocking mode and copying everything to the socket */
	static int stdio_read_thread(void *data)
	{
		int len;
		char buff[4096];

		for(;;) {
			len = read(0, buff, sizeof(buff)-1);
			if (len > 0) {
				buff[len] = '\0';
				send(stdin_sock, buff, len, 0);
			}
		}
		return 0;
	}

	/* thread for reading the socket and copying everything to stdout */
	static int stdio_write_thread(void *data)
	{
		int len;
		char buff[4096];
		struct P_pollfd fds[1];

		fds[0].fd = stdout_sock;
		fds[0].events = P_POLLIN;
		for(;;) {
			P_poll(fds, 1, -1);
			if (fds[0].revents & P_POLLIN) {
				len = recv(stdout_sock, buff, sizeof(buff), 0);
				if (len > 0)
					write(1, buff, len);
			}
		}
		return 0;
	}

	int P_net_async_stdio(P_net_socket *fd_stdin, P_net_socket *fd_stdout)
	{
		int ret;
		ret = P_net_local_socket(fd_stdin, &stdin_sock, P_net_nonblock_full);
		if (ret != 0)
			return ret;
		ret = P_net_local_socket(fd_stdout, &stdout_sock, P_net_nonblock_full);
		if (ret != 0)
			return ret;
		P_bgtask_create(stdio_read_thread, NULL);
		P_bgtask_create(stdio_write_thread, NULL);
		return 0;
	}

#else
	int P_net_async_stdio(P_net_socket *fd_stdin, P_net_socket *fd_stdout)
	{
		*fd_stdin = 0;
		*fd_stdout = 1;
		P_net_set_nonblocking(0);
		P_net_set_nonblocking(1);
		return 0;
	}
#endif
