#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "c99tree-cc.h"
#include "tree_load.h"
#include "tree_print.h"
#include "escaping.h"

char *filename;
char *window_open = "`";
char *window_close = "`";
int WINDOW = 16;
c99tree_cc_t ctx;

int inp_file_size_(c99c_node_t *root, int top_size)
{
	c99c_node_t *n;

	if (root->loc.eoffs > top_size)
		top_size = root->loc.eoffs;

	for(n = root->children; n != NULL; n = n->next) {
		top_size = inp_file_size_(n, top_size);
	}

	return top_size;
}

int inp_file_size(c99c_node_t *root)
{
	int top_size;
	c99c_node_t *n;

	top_size = 0;
	for(n = root; n != NULL; n = n->next)
		top_size = inp_file_size_(n, top_size);

	return top_size+32; /* leave some room for trailing ws and } */
}

char *src;
int src_size;
void load_src(const char *fn, int size)
{
	int fd;

	fd = open(fn, O_RDONLY);
	if (fd < 0) {
		fprintf(stderr, "failed to open %s\n", fn);
		exit(1);
	}

	src = malloc(size);
	src_size = read(fd, src, size);
	if (src_size <= 0) {
		fprintf(stderr, "failed to read %s\n", fn);
		exit(1);
	}
	
	close(fd);
}

void print_src(int from, int to)
{
	int n, f, t;
	char *s;

	for(n = 0, f = from; n < WINDOW; n++,f--)
		if ((f == 0) || (src[f] == '\n'))
			break;

	if (f < from)
		f++;


	for(n = 0, t = to; n < WINDOW; n++,t++)
		if ((t == src_size-1) || (src[t] == '\n'))
			break;

	if (t > to)
		t--;

	for(s = src+f, n = f; n <= t; n++,s++) {
		if (n == from)
			printf("%s", window_open);
		switch(*s) {
			case '\t': printf(" "); break;
			case '\n': printf("\\n"); break;
			case '\r': printf("\\r"); break;
			default: printf("%c", *s);
		}
		if (n == to)
			printf("%s", window_close);
	}
}

int Nprint_ind_src(c99tree_cc_t *ctx, void **nprint_ctx, const char *prefix, const c99c_node_t *node, print_state_t state)
{
	char *tmp;
	int len;
	char buf[1024];

	switch(state) {
		case PS_OPENING:
			tmp = c99c_escape(node->name);
			len = printf("%s%s %s (%s)", prefix, c99c_node_type_names[node->type], tmp, c99c_loc_print(ctx, node, buf, sizeof(buf)));
			while(len < 40) {
				printf(" ");
				len++;
			}
			if ((node->loc.is_local) && (node->loc.soffs >= 0) && (node->loc.eoffs >= 0))
				print_src(node->loc.soffs, node->loc.eoffs);
			printf("\n");
			free(tmp);
			break;
		case PS_HEAD:
			printf("FILE  (%s)\n", ctx->loc_format);
			break;
		case PS_TAIL:
		case PS_CLOSING:
			break;
	}
	return 0;
}



int main(int argc, char *argv[])
{
	char *fn_ind = NULL, *fn_c = NULL;
	c99c_node_t *root;
	int n;
	char *cmd, *arg;
	void *nprint_ctx = NULL;

	ctx.loc_format = "%o:%O";
	for(n = 1; n < argc; n++) {
		cmd = argv[n];
		arg = argv[n+1];
		if (*cmd != '-')
			goto badarg;
		while(*cmd == '-')
			cmd++;
		switch(*cmd) {
			case 'i': fn_ind = arg; n++; break;
			case 'c': fn_c = arg; n++; break;
			case 'O': window_open = cmd+1; break;
			case 'C': window_close = cmd+1; break;
			case 'L': ctx.loc_format = cmd+1; break;
			default:
				badarg:;
				fprintf(stderr, "Unrecognized command line argument: '%s'\n", cmd);
				exit(1);
		}
	}


	c99c_tree_init(&ctx);

	root = c99c_tree_load_ind(&ctx, fn_ind);
	load_src(fn_c, inp_file_size(root));

	ctx.Nprint_cb = Nprint_ind_src;

	ctx.Nprint_cb(&ctx, &nprint_ctx, "", root, PS_HEAD);
	C99Nprint(&ctx, &nprint_ctx, " ", root, 0);
	ctx.Nprint_cb(&ctx, &nprint_ctx, "", root, PS_TAIL);

	return 0;
}
